//****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<com.ibm.etools.cobol>>
//   (C) Copyright IBM Corp. 2003,2004 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************

package com.ibm.etools.cobol.importer;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;

import com.ibm.etools.cobol.plugin.CobolPlugin;
import com.ibm.etools.logging.util.Level;

/**
 * This class responsible for loading the dll(Runcommand.dll, thats 
 * contains the C code that contains the native method implementation 
 * runCommand method) and pass the parameters thats are required by
 * the C code to invoke the compiler for Syntax check and creation of
 * XMI.
 * @author: Mahendra Akkina
 */
public class RunCommand implements ICobolPreferenceConstants {
	private HashMap fpfs = new HashMap();
	private HashMap eds = new HashMap();
	private String path, workingDir, command, cobolimporterLocation, exitRoutinesLocation, localePathLocation;
	private String platform, codePageSelection, floatingPointFormat, bigEndian, remoteIntegerEndian,
					externalDecimalSign, quote, trunc, nsymbol, xmiOptions, otherOptions, 
					compileTimeLocale, asciiCodepage, errorMsgLang, currencySign, collseq, ncollseq;
	private HashMap extensionSupportMap = new HashMap();
	private static String additionalCOPYBookLocations="";
					            					
	/**
	 * Native method signature
	 */	
    public native void runCommand(String path, String workingDir, String command, String cobolimporterLocation, String exitRoutinesLocation, String localePathLocation, String additionalCOPYBookLocations, String locale, String errorMsgLang, boolean isTracingOn);

	//Load the dll
    static {
        System.load(CobolPlugin.getPluginLocation("com.ibm.etools.cobol") + "importer/dll/RunCommand.dll");
        additionalCOPYBookLocations = CobolPlugin.getAdditionalCOPYBookLocations();        
        CobolPlugin.getPlugin().writeMsg(Level.FINE,"additionalCOPYBookLocations = " + additionalCOPYBookLocations);
    }

	/**
	 * Constructor.
	 * @param path - location of the selected file
	 * @param workingDir - temporary working directory
	 */
	public RunCommand(String path, String workingDir) {
		this.path = path;
		this.workingDir =  workingDir;
		fpfs.put(new Integer(0),"ieeeNonExtended");
		fpfs.put(new Integer(1),"ibm390Hex");
		eds.put(DECIMAL_ASCII,"ascii");
		eds.put(DECIMAL_EBCDIC,"ebcdic");
		eds.put(DECIMAL_EBCDICC,"ebcdicCustom");
		//assign default values
		quote="QUOTE";
		trunc=TRUNC_STD;
		nsymbol=NSYMBOL_NATIONAL;
		collseq=COLLSEQ_BIN;
		ncollseq=NCOLLSEQ_BIN;
		xmiOptions="";
		//All compiler option that result in listing output are turned off (Also add arith(extend) to be more liberal)
		otherOptions="NOXREF,NOMAP,NOLIST,NOSOURCE,NOCOMPILE,NOSEQ,NONUM,FLAG(W),NOTERM,arith(extend),NOSQL,NOCICS";

		//Get the location of the COBOL Compiler components
		cobolimporterLocation = CobolPlugin.getPluginLocation("com.ibm.etools.cobol") + "importer/cobol";
		exitRoutinesLocation = CobolPlugin.getPluginLocation("com.ibm.etools.cobol") + "importer/exit_routines";
		localePathLocation = CobolPlugin.getPluginLocation("com.ibm.etools.cobol") + "importer/locale";
	}

	/**
	 * Get the options, created the command string thats is passed to 
	 * native C code.
	 */
    public void createXMI(HashMap props) throws CobolException{
    	int asciiCodepageIndex = 0; 
    	COBOLLocaleRelatedConstants clrc = new COBOLLocaleRelatedConstants();
        CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): cobol options = " + props);     		
		if (props != null) {
			Iterator it = props.keySet().iterator();
			while (it.hasNext()) {
				String key = (String)it.next();
				if (key.equals(PRE_COBOL_PLATFORM_SELECTION)){
					//need to know what to do with this?
					platform = ((Integer)props.get(key)).toString();
				} else if (key.equals(PRE_COBOL_CODEPAGE)){
					codePageSelection = (String)props.get(key);
					xmiOptions = xmiOptions.concat("CodePage=" + codePageSelection + ",");
				} else if (key.equals(PRE_COBOL_FLOATING_POINT_FORMAT)){
					floatingPointFormat = (String)fpfs.get((Integer)props.get(key));
					xmiOptions = xmiOptions.concat("FloatingPoint=" + floatingPointFormat + ",");
				} else if (key.equals(PRE_COBOL_ENDIAN)){
					if (((String)props.get(key)).equals(ENDIAN_BIG))
						bigEndian = "true";
					else
						bigEndian = "false";
					xmiOptions = xmiOptions.concat("BigEndian=" + bigEndian + ",");
				} else if (key.equals(PRE_COBOL_REMOTE_ENDIAN)){
					//needs to be resolved
					remoteIntegerEndian = (String)props.get(key);
				} else if (key.equals(PRE_COBOL_EXT_DECIMAL_SIGN)){
					externalDecimalSign = (String)eds.get((String)props.get(key));
					xmiOptions = xmiOptions.concat("EDSign=" + externalDecimalSign + ",");
					/* 
					 * If ebcdic custom is selected and host code page value is
					 * other than IBM-037, add it to the xmiOptions
					 */ 
					if (((String)props.get(key)).equals(DECIMAL_EBCDICC)){
						if (!((String)props.get(PRE_COBOL_HOST_CODEPAGE_EC)).equals(OS_CODEPAGE)) {
							xmiOptions = xmiOptions.concat("HostCodePage=" + (String)props.get(PRE_COBOL_HOST_CODEPAGE_EC) + ",");
						}
					}
				} else if (key.equals(PRE_COBOL_QUOTE)){
					if (((String)props.get(key)).equals(QUOTE_SINGLE))
						quote = "APOST";
					otherOptions = otherOptions.concat("," + quote);
				} else if (key.equals(PRE_COBOL_TRUNC)){
					if (((String)props.get(key)).equals(TRUNC_OPT))
						trunc = TRUNC_OPT;
					else if (((String)props.get(key)).equals(TRUNC_BIN))
						trunc = TRUNC_BIN;
					otherOptions = otherOptions.concat(",TRUNC("+trunc+")");
				} else if (key.equals(PRE_COBOL_NSYMBOL)){
					if (((String)props.get(key)).equals(NSYMBOL_DBCS))
						nsymbol=NSYMBOL_DBCS;
					otherOptions = otherOptions.concat(",NSYMBOL("+nsymbol+")");
				} else if (key.equals(PRE_COBOL_COMPILE_TIME_LOCALE)){	
					compileTimeLocale = (String) clrc.getLocaleNames().get(((Integer) props.get(key)).intValue());
				} else if (key.equals(PRE_COBOL_ASCII_CODEPAGE)){
					asciiCodepageIndex = ((Integer) props.get(key)).intValue();
				} else if (key.equals(PRE_COBOL_ERROR_MSGS_LANG)) {
					errorMsgLang = (String) clrc.getErrorMessagesLanguage().get(((Integer) props.get(key)).intValue());
				} else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_CURRENCY)) {
					currencySign =  ((String)props.get(key));					
					currencySign = currencySign.replace('"','\'');					
					otherOptions = otherOptions.concat(",CURRENCY("+currencySign+")");					
				} else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_SOSI)) {
					if (((Boolean) props.get(key)).booleanValue())
						otherOptions = otherOptions.concat(",SOSI");
				} else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_COLLSEQ)) {
					if (((String)props.get(key)).equals(COLLSEQ_EBCDIC))
						collseq = COLLSEQ_EBCDIC;
					else if (((String)props.get(key)).equals(COLLSEQ_LOCALE))
						collseq = COLLSEQ_LOCALE;
					otherOptions = otherOptions.concat(",COLLSEQ("+collseq+")");
				} else if (key.equals(ICobolPreferenceConstants.PRE_COBOL_NCOLLSEQ)) {
					if (((String)props.get(key)).equals(NCOLLSEQ_LOCALE))
						ncollseq = NCOLLSEQ_LOCALE;
					otherOptions = otherOptions.concat(",NCOLLSEQ("+ncollseq+")");					
				} else if (key.equals(PRE_COBOL_EXTENSION_CBL)) {
					extensionSupportMap.put(".cbl", (String) props.get(key));
				} else if (key.equals(PRE_COBOL_EXTENSION_CCP)) {
					extensionSupportMap.put(".ccp", (String) props.get(key));
				} else if (key.equals(PRE_COBOL_EXTENSION_COB)) {
					extensionSupportMap.put(".cob", (String) props.get(key));
				} else if (key.equals(PRE_COBOL_EXTENSION_CPY)) {
					extensionSupportMap.put(".cpy", (String) props.get(key));
				} else {
                    CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): Ignored Key:Value = " + key + ":" + props.get(key)); 					
				}
			}
			asciiCodepage = (String) (clrc.getASCIICodePages(compileTimeLocale).get(asciiCodepageIndex));
		}
		String filePath = path;
		path = path.replace('\\','/');
		String fileName = path.substring(path.lastIndexOf('/')+1, path.length());		
		String fileNameWOX = fileName.substring(0, fileName.lastIndexOf('.'));	
		path = path.substring(0, path.lastIndexOf('/'));
		CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): filename =" + fileName); 							
		CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): filename w/o extension =" + fileNameWOX); 					
		CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): path(current dir)="+path); 	
		CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): workingDir="+workingDir); 	
		CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): cobolimporterLocation="+cobolimporterLocation);		
								
		if (xmiOptions.endsWith(","))
			xmiOptions = xmiOptions.substring(0, xmiOptions.length()-1);
		//iLibs: For Syslib.. Need to decide if we are planning to support COPY Books existing in other than current directory.
		//String iLibs = path;
		String ext = fileName.substring(fileName.lastIndexOf('.'), fileName.length()).toLowerCase();
		String prtExit = ",prtexit(iwzprtx)";
		if (CobolPlugin.getMsgLogger().getLevel() <= Level.FINE)		
			prtExit = "";
		if (extensionSupportMap.get(ext).equals(ICobolPreferenceConstants.NOT_DEFAULT_ES)) //Data Structures only
			command = "-q\"XMI('"+xmiOptions+"'),ADATA,exit(inexit('"+fileName+"',iwzinx),xmiexit('cobol.xmi',iwzxmix),adexit(iwzadx)"+prtExit+"),"+otherOptions+"\"";
		else
			command = "-q\"XMI('"+xmiOptions+"'),ADATA,exit(xmiexit('cobol.xmi',iwzxmix),adexit(iwzadx)"+prtExit+"),"+otherOptions+"\" \"" + filePath +"\"";			
		if (command == null) {
   			CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** RunCommand.createXMI(Hashmap): " + CobolResource.getString("_File_Extension_Not_Supported")); 				
            throw new CobolException(CobolResource.getString("_File_Extension_Not_Supported"));
        } else {
   			CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): command=" + command);
   			if (CobolPlugin.getMsgLogger().getLevel() <= Level.FINE) {
				CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createXMI(Hashmap): c tracing= TRUE");
				createLoggingStateFile();
     			runCommand(path, workingDir, command, cobolimporterLocation, exitRoutinesLocation, localePathLocation, additionalCOPYBookLocations, compileTimeLocale + "." + asciiCodepage, errorMsgLang, true);
   			} else
     			runCommand(path, workingDir, command, cobolimporterLocation, exitRoutinesLocation, localePathLocation, additionalCOPYBookLocations, compileTimeLocale + "." + asciiCodepage, errorMsgLang, false);
		}
	}

	/**
	 * This method is used to create a logging file which is used in the exit routines code
	 * to determine - to log the traces or not.
	 * @param string
	 */
	private void createLoggingStateFile() {
		CobolPlugin.getPlugin().writeMsg(Level.FINE,"*** RunCommand.createLoggingFileForExitRoutines(String)");	
		java.io.File loggingStateFile = new java.io.File(workingDir + "\\cobol_exit_routines.log");
		try {
			loggingStateFile.createNewFile();					
		} catch(IOException ioe) { 
			CobolPlugin.getPlugin().writeMsg(Level.SEVERE,"*** RunCommand.createLoggingFileForExitRoutines(String): IOException occured - " + ioe.getMessage(), ioe);					
		}
		
	}

	public HashMap getExtensionSupportMap() {
		return extensionSupportMap;
	}
}